/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace MobiServer.Devices.Phidget.Data
{

    public class Types
    {
        public const string Accel = "PhidgetAccelerometer";
        public const string InterfaceKit = "PhidgetInterfaceKit";
        public const string Weight = "PhidgetWeightSensor";
        public const string RFID = "PhidgetRFID";
        public const string Servo = "PhidgetServo";
        public const string TextLCD = "PhidgetTextLCD";
        public const string PH = "PhidgetPHSensor";
        public const string Motor = "PhidgetMotorControl";
        public const string Temperature = "PhidgetTemperatureSensor";
        public const string Encoder = "PhidgetEncoder";
    }

    public class States
    {
        public const string In = "In";
        public const string Out = "Out";
        public const string Attach = "Attach";
        public const string Detach = "Detach";
        public const string Set = "Set";
    }

    public class ShortTypes
    {
        public const string Weight = "Weight";
        public const string InterfaceKit = "InterfaceKit";
        public const string RFID = "RFID";
        public const string Accel = "Accel";
        public const string Servo = "Servo";
        public const string TextLCD = "TextLCD";
        public const string PH = "PH";
        public const string Motor = "Motor";
        public const string Temperature = "Temperature";
        public const string Encoder = "Encoder";
    }

    public class Base
    {
        public const string HEADER = "Phidgets";
        public const int ARGS_LENGTH_MIN = 4;

        protected string type = null;
        protected string shortType = null;
        protected string state;
        protected int serialNumber = 0;

        public Base(string type, int serialNumber, string state)
        {
            this.type = GetLongType(type);
            this.shortType = GetShortType(this.type);
            this.serialNumber = serialNumber;
            this.state = GetState(state);
        }

        private string GetLongType(string type)
        {
            Regex regex = new Regex(type, RegexOptions.IgnoreCase);

            if (regex.Match(ShortTypes.Accel).Success)
                type = Types.Accel;
            else if (regex.Match(ShortTypes.Encoder).Success)
                type = Types.Encoder;
            else if (regex.Match(ShortTypes.InterfaceKit).Success)
                type = Types.InterfaceKit;
            else if (regex.Match(ShortTypes.Motor).Success)
                type = Types.Motor;
            else if (regex.Match(ShortTypes.PH).Success)
                type = Types.PH;
            else if (regex.Match(ShortTypes.RFID).Success)
                type = Types.RFID;
            else if (regex.Match(ShortTypes.Servo).Success)
                type = Types.Servo;
            else if (regex.Match(ShortTypes.Temperature).Success)
                type = Types.Temperature;                            
            else if (regex.Match(ShortTypes.TextLCD).Success)
                type = Types.TextLCD;
            else if (regex.Match(ShortTypes.Weight).Success)
                type = Types.Weight;

            return type;
        }

        private string GetShortType(string type)
        {

            Regex regex = new Regex(type, RegexOptions.IgnoreCase);

            if (regex.Match(Types.Accel).Success)
                type = ShortTypes.Accel;
            else if (regex.Match(Types.Encoder).Success)
                type = ShortTypes.Encoder;
            else if (regex.Match(Types.InterfaceKit).Success)
                type = ShortTypes.InterfaceKit;
            else if (regex.Match(Types.Motor).Success)
                type = ShortTypes.Motor;
            else if (regex.Match(Types.PH).Success)
                type = ShortTypes.PH;
            else if (regex.Match(Types.RFID).Success)
                type = ShortTypes.RFID;
            else if (regex.Match(Types.Servo).Success)
                type = ShortTypes.Servo;
            else if (regex.Match(Types.Temperature).Success)
                type = ShortTypes.Temperature;
            else if (regex.Match(Types.TextLCD).Success)
                type = ShortTypes.TextLCD;
            else if (regex.Match(Types.Weight).Success)
                type = ShortTypes.Weight;

            return type;
        }

        private string GetState(string state)
        {
            Regex regex = new Regex(state, RegexOptions.IgnoreCase);

            if (regex.Match(States.Attach).Success)
                state = States.Attach;
            else if (regex.Match(States.Detach).Success)
                state = States.Detach;
            else if (regex.Match(States.In).Success)
                state = States.In;
            else if (regex.Match(States.Out).Success)
                state = States.Out;
            else if (regex.Match(States.Set).Success)
                state = States.Set;

            return state;
        }

        protected string [] GetHeader()
        {
            string[] args = { HEADER, state, shortType, serialNumber.ToString() };
            return args;
        }

        protected static string ParseState(string [] args)
        {
            return args[1];
        }

        protected static int ParseSerialNumber(string[] args)
        {
            return int.Parse(args[3]);
        }

        protected static string ParseType(string[] args)
        {
            return args[2];

        }

        public string [] ToArray()
        {
            string[] args = null;
            switch (state)
            {
                case States.Attach:
                case States.Detach:
                    args = GetHeader();
                    break;
            }
            return args;
        }
        
        public static Base Parse(string [] args)
        {
            Base phidgetData = null;
            const int ARGS_LENGTH_MIN = 4;
            string type = "";
            int serialNumber = 0;
            string state = "";

            if (args.Length >= ARGS_LENGTH_MIN && args[0] == HEADER)
            {
                try
                {
                    type = ParseType(args);
                    serialNumber =ParseSerialNumber(args);
                    state = ParseState(args);
                    phidgetData = new Base(type, serialNumber, state);
                }
                catch(Exception ex)
                {
            
                }
            }
          
            return phidgetData;
        }


   
        public string State
        {
            get { return this.state; }
        }

        public string Type
        {
            get { return this.type; }
        }

        public int SerialNumber
        {
            get { return this.serialNumber; }
        }
    }
}
